﻿using UnityEngine;

public class StatModifier : Mod
{
    static bool saving = false;
    static string fileName
    {
        get
        {
            string time = "";
            if (saving)
                time = System.DateTime.Now.ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            else
                time = new System.DateTime(SaveAndLoad.WorldToLoad.lastPlayedDateTicks).ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            string path = System.IO.Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, time);
            if (System.IO.Directory.Exists(path))
                return System.IO.Path.Combine(path,"StatModifier.json");
            return System.IO.Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, time + SaveAndLoad.latestStringNameEnding, "StatModifier.json");
        }
    }
    static float[,] oldValues = null;
    private static float modiFValue = 1f;
    private static float modiWValue = 1f;
    static float currentFoodModifier
    {
        get
        {
            return modiFValue;
        }
        set {
            modiFValue = value;
            HarmonyLib.Traverse fetcher = HarmonyLib.Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
            SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
            for (int i = 0;i < data.Length;i++)
            {
                data[i].nourishmentVariables.foodDecrementRateMultiplier = oldValues[i, 0] * modiFValue;
            }
        }

    }
    static float currentWaterModifier
    {
        get
        {
            return modiWValue;
        }
        set
        {
            modiWValue = value;
            HarmonyLib.Traverse fetcher = HarmonyLib.Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
            SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
            for (int i = 0; i < data.Length; i++)
            {
                data[i].nourishmentVariables.thirstDecrementRateMultiplier = oldValues[i, 1] * modiWValue;
            }
        }

    }
    public void Start()
    {
        HarmonyLib.Traverse fetcher = HarmonyLib.Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
        SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
        oldValues = new float[data.Length, 2];
        for (int i = 0; i < data.Length; i++)
        {
            oldValues[i, 0] = data[i].nourishmentVariables.foodDecrementRateMultiplier;
            oldValues[i, 1] = data[i].nourishmentVariables.thirstDecrementRateMultiplier;
        }
        Log("Mod has been loaded!");
    }

    [ConsoleCommand(name: "statmodifier", docs: "Changes or returns current stat modifier")]
    public static string MyCommand(string[] args)
    {
        if (args.Length == 0)
            return "Hunger: " + currentFoodModifier.ToString() + "\nThirst: " + currentWaterModifier.ToString();
        if (args.Length > 1) 
            return "Too many arguments";
        try
        {
            float tmp = float.Parse(args[0]);
            currentFoodModifier = tmp;
            currentWaterModifier = tmp;
            return "Stat modifiers are now " + tmp.ToString();
        } catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:food", docs: "Changes or returns current hunger stat modifier")]
    public static string MyCommand2(string[] args)
    {
        if (args.Length == 0)
            return currentFoodModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentFoodModifier = float.Parse(args[0]);
            return "Hunger stat modifier is now " + currentFoodModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:water", docs: "Changes or returns current thirst stat modifier")]
    public static string MyCommand3(string[] args)
    {
        if (args.Length == 0)
            return currentWaterModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentWaterModifier = float.Parse(args[0]);
            return "Thirst stat modifier is now " + currentWaterModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    override public void WorldEvent_WorldSaved()
    {
        saving = true;
        JSONObject customSaveData = getSaveJson();
        if (customSaveData.HasField("Modifier_Food"))
        {
            customSaveData.SetField("Modifier_Food", currentFoodModifier);
        }
        else
        {
            customSaveData.AddField("Modifier_Food", currentFoodModifier);
        }
        if (customSaveData.HasField("Modifier_Water"))
        {
            customSaveData.SetField("Modifier_Water", currentWaterModifier);
        }
        else
        {
            customSaveData.AddField("Modifier_Water", currentWaterModifier);
        }
        saveJson(customSaveData);
        saving = false;
    }

    public override void WorldEvent_WorldLoaded()
    {
        JSONObject customSaveData = getSaveJson();
        float currentModifier = 1;
        if (customSaveData.HasField("Modifier"))
        {
            customSaveData.GetField(ref currentModifier ,"Modifier");
            customSaveData.RemoveField("Modifier");
        }
        if (customSaveData.HasField("Modifier_Food"))
        {
            currentFoodModifier = customSaveData.GetField("Modifier_Food").n;
        }
        else
        {
            currentFoodModifier = currentModifier;
        }
        if (customSaveData.HasField("Modifier_Water"))
        {
            currentWaterModifier = customSaveData.GetField("Modifier_Water").n;
        }
        else
        {
            currentWaterModifier = currentModifier;
        }
    }

    public void OnModUnload()
    {
        HarmonyLib.Traverse fetcher = HarmonyLib.Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
        SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
        for (int i = 0; i < data.Length; i++)
        {
            data[i].nourishmentVariables.foodDecrementRateMultiplier = oldValues[i, 0];
            data[i].nourishmentVariables.thirstDecrementRateMultiplier = oldValues[i, 1];
        }
        Log("Mod has been unloaded!");
    }

    private JSONObject getSaveJson()
    {
        JSONObject data;
        try
        {
            data = new JSONObject(System.IO.File.ReadAllText(fileName));
        } catch
        {
            data = JSONObject.Create();
            saveJson(data);
        }
        return data;
    }

    private void saveJson(JSONObject data)
    {
        try
        {
            if (!data.IsNull)
            System.IO.File.WriteAllText(fileName, data.ToString());
        } catch (System.Exception err)
        {
            Log("An error occured while trying to save settings: " + err.Message);
        }
    }

    public static void Log(object message)
    {
        Debug.Log("[StatModifier]: " + message.ToString());
    }
}